@@ -153,7 +153,7 @@ enter_file (const char *name)
   struct file file_key;
 
   assert (*name != '\0');
-  assert (strcache_iscached (name));
+  assert (! verify_flag || strcache_iscached (name));
 
 #if defined(VMS) && !defined(WANT_CASE_SENSITIVE_TARGETS)
   if (*name != '.')
@@ -184,7 +184,7 @@ enter_file (const char *name)
 
   new = xcalloc (sizeof (struct file));
   new->name = new->hname = name;
-  new->update_status = -1;
+  new->update_status = us_none;
 
   if (HASH_VACANT (f))
     {
@@ -378,7 +378,7 @@ remove_intermediates (int sig)
             && !f->secondary && !f->cmd_target)
           {
             int status;
-            if (f->update_status == -1)
+            if (f->update_status == us_none)
               /* If nothing would have created this file yet,
                  don't print an "rm" command for it.  */
               continue;
@@ -430,7 +430,8 @@ remove_intermediates (int sig)
 struct dep *
 split_prereqs (char *p)
 {
-  struct dep *new = PARSE_FILE_SEQ (&p, struct dep, '|', NULL, 0);
+  struct dep *new = PARSE_FILE_SEQ (&p, struct dep, MAP_PIPE, NULL,
+                                    PARSEFS_NONE);
 
   if (*p)
     {
@@ -439,7 +440,7 @@ split_prereqs (char *p)
       struct dep *ood;
 
       ++p;
-      ood = PARSE_FILE_SEQ (&p, struct dep, '\0', NULL, 0);
+      ood = PARSE_SIMPLE_SEQ (&p, struct dep);
 
       if (! new)
         new = ood;
@@ -1000,23 +1001,18 @@ print_file (const void *item)
     case cs_finished:
       switch (f->update_status)
         {
-        case -1:
+        case us_none:
           break;
-        case 0:
+        case us_success:
           puts (_("#  Successfully updated."));
           break;
-        case 1:
+        case us_question:
           assert (question_flag);
           puts (_("#  Needs to be updated (-q is set)."));
           break;
-        case 2:
+        case us_failed:
           puts (_("#  Failed to be updated."));
           break;
-        default:
-          puts (_("#  Invalid value in 'update_status' member!"));
-          fflush (stdout);
-          fflush (stderr);
-          abort ();
         }
       break;
     default:
@@ -1052,7 +1048,7 @@ print_file_data_base (void)
 #define VERIFY_CACHED(_p,_n) \
     do{\
         if (_p->_n && _p->_n[0] && !strcache_iscached (_p->_n)) \
-          error (NULL, "%s: Field '%s' not cached: %s\n", _p->name, # _n, _p->_n); \
+          error (NULL, _("%s: Field '%s' not cached: %s"), _p->name, # _n, _p->_n); \
     }while(0)
 
 static void
